@@ -98,7 +98,7 @@ ComputeNtrPosteriorUpper=function(data,CI.size=0.95,name="upper") ComputeNtrPost
 #' @export
 #'
 #' @concept snapshot
-ComputeSteadyStateHalfLives=function(data,time=Design$dur.4sU,name, columns=NULL, max.HL=48, CI.size=0.95, compute.CI=FALSE, as.analysis=FALSE) {
+ComputeSteadyStateHalfLives=function(data,time=Design$dur.4sU,name="HL", columns=NULL, max.HL=48, CI.size=0.95, compute.CI=FALSE, as.analysis=FALSE) {
   if (is.character(time) && length(time)==1) time=Coldata(data,time)
 
   ntrs=as.matrix(GetTable(data,type="ntr",name.by = "Gene"))
@@ -156,6 +156,7 @@ ComputeSteadyStateHalfLives=function(data,time=Design$dur.4sU,name, columns=NULL
 #'
 #' @concept preprocess
 ComputeAbsolute=function(data,dilution=4E4,volume=10,slot="tpm",name="absolute") {
+  checkPackages(c("monocle","VGAM"))
   fd=data.frame(gene_short_name=Genes(data))
   rownames(fd)=Genes(data)
   mat=GetTable(data,type=slot)
@@ -390,14 +391,14 @@ FilterGenes=function(data,mode.slot='count',minval=100,mincol=ncol(data)/2,min.c
   if (is.null(use)) {
     summi = if (!is.null(min.cond)) GetSummarizeMatrix(data,no4sU=TRUE,average=FALSE) else NULL
     mincol=if (!is.null(min.cond)) min.cond else mincol
-    t=GetTable(data,type=mode.slot,summarize = summi)
-    use=apply(t,1,function(v) sum(v>=minval,na.rm=TRUE)>=mincol)
+    t=GetMatrix(data,mode.slot=mode.slot,summarize = summi)
+    use=Matrix::rowSums(t>=minval,na.rm=TRUE)>=mincol
+    #use=apply(t,1,function(v) sum(v>=minval,na.rm=TRUE)>=mincol)
     if (!is.null(keep)) use = use | rownames(t) %in% rownames(t[keep,])
-  } else {
-    use=ToIndex(data,use)
   }
+  use=ToIndex(data,use)
 
-  if (return.genes) return(Genes(data,use))
+  if (return.genes) return(unname(use))
   return(data.apply(data,function(t) t[use,],fun.gene.info = function(t) t[use,]))
 }
 
@@ -410,10 +411,16 @@ FilterGenes=function(data,mode.slot='count',minval=100,mincol=ncol(data)/2,min.c
 #' @param name the new name by which this is added to the Coldata
 #' @param genes define the set of genes to compute the percentage for
 #' @param mode.slot which mode.slot to take the values for computing the percentage from
+#' @param genes.total define the set of genes defining the total value
+#' @param mode.slot.total which mode.slot to take the values for computing the total
 #' @param multiply.by.100 if TRUE, compute percentage values, otherwise fractions between 0 and 1
 #'
 #' @seealso \code{\link{Coldata}}
 #'
+#' @details The percentages are computed for the given genes with the given mode.slot, w.r.t the mode.slot.total from the genes.total. Thus
+#' to compute the percentage of mitochondrial gene expression in total RNA (unnormalized), only set genes=Genes(data,"^MT-",regex=TRUE).
+#' To compute the percentage of new RNA among all genes, set mode.slot="new.count" and mode.slot.total="count".
+#'
 #' @details Genes can be referred to by their names, symbols, row numbers in the gene table, or a logical vector referring to the gene table rows.
 #'
 #' @details To refer to data slots, the mode.slot syntax can be used: Each name is either a data slot, or one of (new,old,total)
@@ -423,11 +430,46 @@ FilterGenes=function(data,mode.slot='count',minval=100,mincol=ncol(data)/2,min.c
 #' @export
 #'
 #' @concept data
-ComputeExpressionPercentage=function(data,name,genes,mode.slot=DefaultSlot(data),multiply.by.100=TRUE) {
-  gof=colSums(GetTable(data,type=mode.slot,ntr.na = FALSE,genes = genes))
-  total=colSums(GetTable(data,type=mode.slot,ntr.na = FALSE))
+ComputeExpressionPercentage=function(data,name,genes=Genes(data),mode.slot=DefaultSlot(data),genes.total=Genes(data),mode.slot.total=mode.slot,multiply.by.100=TRUE) {
+  gof=Matrix::colSums(GetMatrix(data,mode.slot=mode.slot,genes = genes))
+  total=Matrix::colSums(GetMatrix(data,mode.slot=mode.slot.total,genes = genes.total))
   percentage=gof/total
   if (multiply.by.100) percentage=percentage*100
   Coldata(data,name)=percentage
   data
 }
+
+#' Compute pseudo NTRs from two count matrices
+#'
+#' NTRs can be computed from given new and total counts.
+#'
+#' @param data a grandR object
+#' @param new.slot the slot containing new RNA counts
+#' @param total.slot the slot containing total RNA counts
+#' @param detection.rate the detection rate of T-to-C mismatch reads (see details)
+#'
+#' @details To correct for some bias, a detection rate (as suggested by Cao et al., Nature Biotech 2020) should be provided. This detection rate
+#' defines, how much new RNA is detected on average using the T-to-C mismatch reads.
+#'
+#' @return a new grandR object
+#' @export
+#'
+#' @concept data
+ComputePseudoNtr=function(data,new.slot,total.slot=DefaultSlot(data),detection.rate=1) {
+  if (!check.slot(data,new.slot,allow.ntr=FALSE) || !check.slot(data,total.slot,allow.ntr=FALSE)) stop("Slot unknown!")
+  n=GetMatrix(data,mode.slot=new.slot,name.by="Gene")
+  t=GetMatrix(data,mode.slot=total.slot,name.by="Gene")
+
+  if (is.matrix(t)) {
+    ntr=pmin((n*detection.rate)/t,1)
+    ntr[is.nan(ntr)]=0
+  } else {
+    sX=Matrix::summary(n)
+    sY=Matrix::summary(t)
+    dd=.Call('fastsparsematdiv',sX$i,sX$j,sX$x,sY$i,sY$j,sY$x,detection.rate)
+
+    ntr=Matrix::sparseMatrix(i=sX$i, j=sX$j, x=dd,dimnames=dimnames(t))
+  }
+
+  AddSlot(data,"ntr",ntr,set.to.default=FALSE)
+}
